// Note the Driver mentioned in the article that maps USB GPS units into
// Windows 8 format is included in this download

main:
  #include "RROScommands.bas"
  gosub InitCommands
  call InitSensors(0,0,0,180)
  // now make time for camera operations (see RROS manual)
  rCommand(SetRROStimeout,180)
  Dim CntInfo[9,9]
  BMP="BMP"
   // wait 3 seconds then take picture
  delay 3000
  for i=1 to 20
    call GetImage("Cam1 10",BMP)
    // display image
    FitBmp "Temp"
  next
  xyString 230,550, "Touch the object to be found"
  repeat
    ReadMouse x,y,b
  until b
  ReadPixel x,y,col
  // set destination manually
  DestLong=-80.4597
  DestLat = 27.6695
  ClearScr
  xyString 100,300, "Move the robot to a remote location and touch the screen."
  // wait for mouse to be released
  repeat
    ReadMouse x,y,b
  until !b
  // and then pressed again
  repeat
    ReadMouse x,y,b
  until b
  // use GPS and compass to move to previous location
  LastLong = DestLong
  LastLat =  DestLat
  done=0
  DirToMove=0
  dif=0
  repeat
    Call ReadGPS(CurLong,CurLat)
    if (CurLong = LastLong) and (CurLat = LastLat)
      // keep moving at DirToMove
      call Orientation(T1,T2,T3,Heading)
      call AngleDifference(Heading,DirToMove,dif)
      if abs(dif)<6 // move forward if on course
        rForward 1
      elseif dif<0 // adjust if necessary
         rCommand(SetTurnStyle,70)
        rTurn -1
      elseif dif>0
         rCommand(SetTurnStyle,70)
        rTurn 1
      endif
    else
      // calculate a new Direction to Move
      call Orientation(T1,T2,T3,Heading)
      call Angle(CurLong,CurLat,DestLong,DestLat,DirToMove,done)
      call AngleDifference(Heading,DirToMove,dif)
      rTurn dif
      LastLong=CurLong
      LastLat=CurLat
    endif
  until done
  // use camera to find the object
  move=0
  cnt=0
  repeat
    call GetImage("Cam1 10",BMP)
    Found=-1
    BMPfindClr "Temp",col,Found,SecCnt,.03,.05,9,CntInfo
    FitBMP "Temp"
    if Found>-1 // if the object is seen
      // determine grid coordinates of block
      x=Found#9
      y=Found/9
      // and move toward object
      if x<4
        rCommand(SetTurnStyle,70)
        rTurn -1
      elseif x>4
        rCommand(SetTurnStyle,70)
        rTurn 1
      else
        rForward 1
      endif
    else // object not seen
      if move=0 
        rTurn -30 // look left
      elseif move=1 
        rTurn 60  // look right
      elseif move=2
        rTurn -30 // back to center
        delay 200       
        rForward 100 // forward a little
        move=0 // start over
        cnt++ // keep track of # of moves
      endif 
      move++     
    endif
  until ((Found>-1) and (x=4) and (rRange()<30)) or (cnt=5)
  rForward 0
  if cnt=5 
    // 5 maneuvers, no object seen
    print "Object not found - I give up"  
  else
   print "Object Found"
  endif   
end

sub Angle(CurLon,CurLat,DestLon,DestLat,&an,&here)
  CurLon=abs(CurLon)
  DestLon=abs(DestLon)
  lon=(CurLon-DestLon)                     
  lat=(CurLat-DestLat)
  here=false
  if lon!=0 then a=rtod(atan(abs(lat/lon)))
  if lon=0 and lat=0 then here=true
  if lon=0 and lat<0 then an=0
  if lon=0 and lat>0 then an=180
  if lon<0 and lat<=0 then an=270+a
  if lon>0 and lat>=0 then an=90+a
  if lon>0 and lat<=0 then an=90-a
  if lon<0 and lat>=0 then an=270-a
return 

sub GetImage(Command,type)  
  FileDelete("Temp.jpg")
  // tell the Utility to take the picture
  SetCBtext(Command) 
  FileRename ("RBsTurn.txt","SensorsTurn.txt")  
  If type="BMP" then FileDelete("Temp.bmp")
  // wait for handshake
  while !FileExists("Temp.jpg")
  wend
  delay 100
  if type="BMP" then ToBmp "Temp.jpg"
return

sub ReadGPS(&a,&b)
   // give the Interface Utility a command
   setcbtext("GPS") 
   // now trigger the Utility to perform 
   filerename ("RBsTurn.txt","SensorsTurn.txt")
   // and wait for it to respond
   while !fileexists("RBsTurn.txt") 
   wend
   delay 200
   // then get the data from the clipboard
   s=getcbtext()
   // and parse it
   mfromstring Sensors,s,"," 
   // and return the values as variables
   a=Sensors[2]
   b=Sensors[1]
   a= Round(a*10000)/10000.0
   b= Round(b*10000)/10000.0
return  

InitRB9robot:
  rCommport 3
  delay 100
  SetTimeOut 50000
  rlocate 0,0 // 255,0
  rCommand(MotorSetup,SMALLDC+ENCODERS)  //+HMC6352 if you have a compass
  // rCommand(SensorSetup,0+PING) //3+FIVERANGE) //FIVERANGE+HMC6352) //0%10001000) rover 5
  rCommand (SensorSetup,SR04+FIVERANGE) // RB-9
  rCommand(SetMotorRamp,10)
  rCommand(SetClicksPerDiam,160)
  rCommand(SetClicksPer90,110) 
  rCommand(SetReducForwRight,0)
  rCommand(SetReducForwLeft,15)
  rCommand(SetReducBackRight,0)
  rCommand(SetReducBackLeft,0)
  rCommand(SetRotationTime,18) // 19)
  rCommand(SetMoveTime,12) //13) 
  // note: speeds were increased for outdoor use
  rCommand(SetSpeed,90) //65) //,35)
  rCommand(SetSlowDownSpeed,85) //45) 
  rCommand(SetSlowDown2,80) 
  rCommand(SetCCdivisor,3)
  rCommand(SetBumpDist,10)
  rCommand(SetProxDist,20)
  rCommand(EnableCounters,1)
  rCommand(SetRobotAngle,90/2)
  rCommand(SetTurnStyle,100)
  rCommand(SetRROStimeout,10)  //15
return

sub InitSensors(a,b,c,d)                     
// The Utility MUST reside in the same directory as
  // the application unless you set up paths 
  print "Initializing..."
  SetCBtext("stop") // stop utility if running
  filerename ("RBsTurn.txt","SensorsTurn.txt")
  delay 100
  Spawn ("RBsensorUtility.exe","",P_NOWAIT) 
  // provide time for Utility to spawn
  delay 1000 
  ClearScr   
  //  The above code can be eliminated if you always
  //  start the Utility manually. We have code in
  //  the utility that SHOULD prevent a second copy
  //  from running - but for some unknown reason it
  //  is not 100% reliable.  The above code seems to
  //  always work on our machine.
  dim Sensors[28] //max available from utility
  dim Offsets[4]                 
  for i=0 to 27
    Sensors[i]=""
  next
  // allow setting certain angles to zero
  Offsets[0]=a
  Offsets[1]=b
  Offsets[2]=c
  Offsets[3]=d 
return


sub Orientation(&t1,&t2,&t3,&h)
   // Request readings
   SetCBtext("tilt")
   filerename ("RBsTurn.txt","SensorsTurn.txt")
   // and wait for it to respond
   while !fileexists("RBsTurn.txt") 
   wend
   delay 50
   // get the data
   s=getcbtext()
   // and parse it
   mfromstring Sensors,s,"," 
   t1 = round(Sensors[1])+Offsets[0]
   t2 = round(Sensors[2])+Offsets[1]
   t3 = round(Sensors[3])+Offsets[2]
   h  = round(Sensors[5])+Offsets[3]
   // ensure heading within normal range
   if h>359 then h-=360
   if h<0 then h+=360
return

sub AngleDifference(a,b,&d)
  // a=current heading, b=destination
  // d=difference (returned value here)
  if a=b
    d=0
    return
  elseif abs(a-b)>180
     if a>b
       b+=360
     else
       a+=360
     endif 
  endif
  d=b-a
return





